! @c Enes Sunel
! This subprogram generates the total deposits distribution over the course of transition and computes sequences of aggregates

subroutine FinDistribution

use parameters
implicit none

! Now we initialize the economy with a total wealth distribution of inf = 14.25% economy and we will compute the "time-variant"
! distributions of total wealth by the help of "time-variant" decision rules.

assetdist = 0
assetdist(:,:,1) =  invasset_0                         ! We start with the ergodic distribution of inf = 14.25% economy
assetdist(:,:,T4) = invasset_T4                        ! Terminal distribution is the ergodic distribution of inf = 2.% economy

! We need to approximate an indicator function for the evolution of total deposits

!!!-----------------------------------------------------------------------------------------------------------
DO t=2,T4-1                                         ! Sequence of total deposits distributions to be computed
!!!-----------------------------------------------------------------------------------------------------------

LL = int( (a_policy_t(:,:,t-1) - alb)/inca) + 1     ! Indices of left shoulders

RR = LL + 1                                         ! Indices of right shoulders

do i=1,na
    do j=1,nx
LW(i,j) = ( asset(RR(i,j),1) - a_policy_t(i,j,t-1) ) / inca   ! Computation of weights for tomorrow's total deposits

if (LW(i,j) > 1.) then
LW(i,j) = 1.
end if

if (LW(i,j) <= 0.) then
LW(i,j) = 0.
end if

    end do
end do

RW = 1. - LW

!!!----------------------------------------------------
! LAW OF MOTION FOR THE DISTRIBUTION OF TOTAL DEPOSITS
!!!----------------------------------------------------

!!! Allocating individuals, depending on the realization of shocks and hh's decisions

do i=1,na
     do j=1,nx
        do jj=1,nx

            assetdist(LL(i,j),jj,t) = assetdist(LL(i,j),jj,t) + LW(i,j)*xprob(j,jj)*assetdist(i,j,t-1)

            assetdist(RR(i,j),jj,t) = assetdist(RR(i,j),jj,t) + RW(i,j)*xprob(j,jj)*assetdist(i,j,t-1)

        end do
     end do
end do


if ( abs(sum(assetdist(:,:,t))-1) > 1.e-04) then

print*,'Distribution in period',t,'does not add up to one!!!',sum(assetdist(:,:,t))

end if

!!!---------------------------------------------------------------------
END DO                          ! Sequence of total deposits distributions is computed
!!!---------------------------------------------------------------------

!!! Since we know the distribution of total deposits, we can compute the aggregates.

L_t(1,1) = L_0
K_t(1,1) = K_0
Y_t(1,1) = Y_0
M_t(1,1) = M_0
B_t(1,1) = B_0
C_t(1,1) =  sum(c_policy_t(:,:,1)*assetdist(:,:,1))
H_t(1,1) = 0

do j=1,nx
    H_t(1,1) = H_t(1,1) + sum(h_policy_t(:,j,1)*assetdist(:,j,1))*x(j,1)
end do

TR_t(1,1)= 0
do i=1,na
    do j=1,nx
        TR_t(1,1) = TR_t(1,1) + assetdist(i,j,1)*phi*c_policy_t(i,j,1)*kap_policy_t(i,j,1)**gama
    end do
end do

!!!---------------------------------------------------------------------
DO t=2,T4-1
!!!---------------------------------------------------------------------

    B_t(1,t) = sum(b_policy_t(:,:,t-1)*assetdist(:,:,t-1))

    M_t(1,t) = sum(m_policy_t(:,:,t-1)*assetdist(:,:,t-1))

    K_t(1,t) = K_a_x_t(1,t-1)

    L_t(1,t) = L_a_x_t(1,t-1)

    Y_t(1,t) = (K_t(1,t)**alpha*L_t(1,t)**(1-alpha))**theta

    C_t(1,t) = sum(c_policy_t(:,:,t)*assetdist(:,:,t))

    H_t(1,t) = 0

    do j=1,nx

        H_t(1,t) = H_t(1,t) + sum(h_policy_t(:,j,t)*assetdist(:,j,t))*x(j,1)

    end do

    Tr_t(1,t)= 0

    do i=1,na
        do j=1,nx

            Tr_t(1,t) = Tr_t(1,t) + assetdist(i,j,t)*phi*c_policy_t(i,j,t)*kap_policy_t(i,j,t)**gama

        end do
    end do


!---------------------------------------------------------------------
END DO                                      ! Aggregates are computed
!---------------------------------------------------------------------

B_t(1,T4) = B_T4
M_t(1,T4) = M_T4
K_t(1,T4) = K_T4
L_t(1,T4) = L_T4
Y_t(1,T4) = Y_T4
C_t(1,T4) = C_T4
H_t(1,T4) = H_T4
G_t(1,T4) = G_T4
TR_t(1,T4)= TR_T4

tau_h_t(1,T4) = tau_hstar_T4        ! Equilibrium labor income taxes for e=2% stationary economy
w_t(1,T4)   = wstar_T4             ! Equilibrium real wage for e=2% stationary economy


! Computing total excess demand in goods and labor market, public surplus
! Updating transfers and real wages

!!!---------------------------------------------------------------------
DO t=1,T4
!!!---------------------------------------------------------------------

    I_t(1,t) = K_t(1,t+1) - (1-delta)*K_t(1,t)
    SE_t(1,t) = M_t(1,t+1) - M_t(1,t)/(1 + inf_t(1,t))

    if (t==1) then

    ED_t(1,t)  = C_t(1,t) + K_t(1,t+1) - (1-delta)*K_t(1,t) + G_t(1,t) + Tr_t(1,t) &

                 + B_t(1,t+1) - R_star_0*B_t(1,t) - Y_t(1,t)

    elseif (t==T4) then

    ED_t(1,t)  = C_t(1,t) + K_T4 - (1-delta)*K_t(1,t) + G_t(1,t) + Tr_t(1,t) &

                 + B_T4 - R_star_t(1,t-1)*B_t(1,t) - Y_t(1,t)
    else

    ED_t(1,t)  = C_t(1,t) + K_t(1,t+1) - (1-delta)*K_t(1,t) + G_t(1,t) + Tr_t(1,t) &

                 + B_t(1,t+1) - R_star_t(1,t-1)*B_t(1,t) - Y_t(1,t)

    end if

    if (t==T4) then

    PS_t(1,t)  = M_T4 - M_t(1,t)/(1 + inf_t(1,t)) &

                + tau_k_t(1,t)*( Y_t(1,t) - delta*K_t(1,t) - w_t(1,t)*L_t(1,t) ) + tau_h_t(1,t)*w_t(1,t)*H_t(1,t) &

                - G_t(1,t) - tau_t(1,t)

    else

    PS_t(1,t)  = M_t(1,t+1) - M_t(1,t)/(1 + inf_t(1,t)) &

                + tau_k_t(1,t)*( Y_t(1,t) - delta*K_t(1,t) - w_t(1,t)*L_t(1,t) ) + tau_h_t(1,t)*w_t(1,t)*H_t(1,t) &

                - G_t(1,t) - tau_t(1,t)

    end if

    L_H_t(1,t) = w_t(1,t)*(L_t(1,t)-H_t(1,t))

    upd_share = 0.75    ! 0.90

    tau_h_t_next(1,t) = tau_h_t(1,t) - (1 - upd_share)*PS_t(1,t)/2

    w_t_next(1,t)   = w_t(1,t) + (1 - upd_share)*L_H_t(1,t)/2

    if ( ((20<t) .and. (t<=30)) .or. (t>=T4-10) ) then

        print*,'---------------------------------------------------------------------'
        print*,'EXCESS DEMAND IN GOODS + LABOR MARKET at time ',t,'is', ED_t(1,t)+L_H_t(1,t)
        print*,'---------------------------------------------------------------------'
        print*,'PUBLIC SURPLUS at time ',t,'is', PS_t(1,t)
        print*,'---------------------------------------------------------------------'
!        print*,'EXCESS DEMAND IN LABOR MARKET at time ',t,'is', L_H_t(1,t)
!        print*,'---------------------------------------------------------------------'
        print*,'Updated taxes at time',t,'is', tau_h_t_next(1,t)
        print*,'---------------------------------------------------------------------'
        print*,'Updated real wages at time',t,'is', w_t_next(1,t)
        print*,'---------------------------------------------------------------------'

    end if

!---------------------------------------------------------------------
END DO                                      ! Candidate transfers are computed
!---------------------------------------------------------------------

!!! Now we store the distance btw the candidate transfers sequence and initial sequence and we update the transfers sequence

tau_h_diff_t = abs(tau_h_t_next - tau_h_t)

wdiff_t   = abs(w_t_next - w_t)

do t = 1,T4

if (PS_t(1,t) == maxval(abs(PS_t(1,:))) ) then
print*,'Maximum PS occurs in period: ',t
end if


if (tau_h_diff_t(1,t) == maxval(abs(tau_h_diff_t(1,:))) ) then
print*,'Maximum tau_h_diff:',tau_h_diff_t(1,t),'occurs in period: ',t
end if

end do

print*,'---------------------------------------------------------------------'
print*,'Update share is:', upd_share!, '(1 - upd_share)*PS_t(1,t)/2'
print*,'-------------------------------------------------------------------------'
print*,'Maximum difference in the taxes sequence is :',maxval(abs(tau_h_diff_t(1,1:T4-1)))
print*,'-------------------------------------------------------------------------'
print*,'Maximum absolute value of public surplus is :',maxval(abs(PS_t(1,1:T4-1)))
print*,'-------------------------------------------------------------------------'
print*,'Maximum difference in the wages sequence is :',maxval(abs(wdiff_t(1,1:T4-1)))
print*,'-------------------------------------------------------------------------'
print*,'Maximum absolute value of excess demand in labor is :',maxval(abs(L_H_t(1,1:T4-1)))
print*,'-------------------------------------------------------------------------'

tau_h_t(1,1:T4-1) =  tau_h_t_next(1,1:T4-1)                      ! Updating the finite sequence of transfers

w_t(1,1:T4-1)   =  w_t_next(1,1:T4-1)                        ! Updating the finite sequence of transfers

end subroutine FinDistribution
