! @c Enes Sunel
! Jan 2015, Central Bank of Turkey
! This subprogram computes the time variant decision rules along the transition path

subroutine FinDecision

use parameters
implicit none

I_bond = 1

b_policy_t = 0
c_policy_t = 0
kap_policy_t = 0
h_policy_t = 0
m_policy_t = 0
a_policy_t = 0
Q_t        = 0
K_a_x_t    = 0
L_a_x_t    = 0
Rk_a_x_t   = 0

!!!--------------------------------------------------------------
! Terminal conditions for decision rules (inf = 2%)
!!!--------------------------------------------------------------

b_policy_t(:,:,T4) = b_policy_T4   ! Time T4 decision rules are the time invariant policy functions and value function of the inf = 2% economy
c_policy_t(:,:,T4) = c_policy_T4
kap_policy_t(:,:,T4) = kap_policy_T4
h_policy_t(:,:,T4) = h_policy_T4
m_policy_t(:,:,T4) = m_policy_T4
a_policy_t(:,:,T4) = a_policy_T4
Q_t(1,T4)          = (alpha/(1-alpha))* w_t(1,T4) / ( (R_star_t(1,T4)-1)/(1-tau_k_t(1,T4)) + delta )
L_a_x_t(1,T4)      = ( w_t(1,T4)/(theta*(1-alpha)*Q_t(1,T4)**(alpha*theta) )  )**(1/(theta-1))
K_a_x_t(1,T4)      = Q_t(1,T4)*L_a_x_t(1,T4)

Vfuture = 0
Vfuture = V_T4                     ! Value in the terminal period corresponds to the time invariant value of inf = 2% economy
Viter  = -1.e+08
Citer  = 0.0001
tol1=1.e-06

scalar1 = 1/(eta-1)

scalar2 = phi*(1+gama)

scalar3 = 1 + gama*varrho

!!!--------------------------------------------------------------
DO  t = T4-1,1,-1                   ! Given the terminal conditions, we solve the decision rules by backward recursion
!!!--------------------------------------------------------------

if (mod(t,20) == 0) then
print*,'-------------------------------------'
print*,'Solving period',t,'decision rules'
print*,'-------------------------------------'
end if


Q_t(1,t) = (alpha/(1-alpha))* w_t(1,t+1) / ( (R_star_t(1,t)-1)/(1-tau_k_t(1,t+1)) + delta )  ! Homogeneous capital-labor demand ratio implied by the real interest rate, real wage rage and capital income tax rate

L_a_x_t(1,t)  = ( w_t(1,t+1)/(theta*(1-alpha)*Q_t(1,t)**(alpha*theta) )  )**(1/(theta-1))    ! Homogeneous labor demand policy function for the given real wage

K_a_x_t(1,t)  = Q_t(1,t)*L_a_x_t(1,t)                                                        ! Homogeneous capital demand policy function for the given real wage

Rk_a_x_t(1,t) = (1-tau_k_t(1,t+1))* ( (K_a_x_t(1,t)**alpha*L_a_x_t(1,t)**(1-alpha))**theta & ! Gross return to capital multiplied by capital

              - delta*K_a_x_t(1,t) - w_t(1,t+1)*L_a_x_t(1,t) ) + K_a_x_t(1,t)

scalar6 =  ( 1 - 1/(R_star_t(1,t)*(1. + inf_t(1,t+1)))  )/(varrho*gama*phi)

    do i=1,na

        scalar4 = asset(i,1) + tau_t(1,t) - K_a_x_t(1,t)

        do j=1,nx

            scalar5 = (1-tau_h_t(1,t))*w_t(1,t)*x(j,1)

            do ii =1,nb

                if (scalar4 + scalar5*0.001  - bond(ii,1) <= 0) then                    ! Infeasible consumption levels for Newton's algorithm

                    cons = 0.0

                else

                cons = scalar4 + scalar5*0.20  - bond(ii,1)
                fc = 5

                do                                                                       ! Solve for consumption in the budget constraint by using Newton's method

                    if ( fc < tol1) exit

                        kap  = cons**((1-varrho)/scalar3)*scalar6**(varrho/scalar3)

                        hour = ( scalar5/( psi*(1+scalar2*kap**gama) ) )**scalar1           ! Computes the labor supply for a given real wage and productivity state

                        fc  = scalar4 + scalar5*hour - bond(ii,1) - (cons**(1+gama)/scalar6)**(1/scalar3) - cons*(1+phi*kap**gama)

                        kap1 = scalar6**(varrho/scalar3)*((1-varrho)/scalar3)*cons**((1-varrho)/scalar3-1)

                        h1 = - scalar1 * hour**(2-eta) * (scalar5/psi) * &

                        (1+scalar2*kap**gama)**(-2) *gama*scalar2* kap**(gama-1)* kap1

                        fc1 = scalar5*h1 - (1/scalar3)*(cons**(1+gama)/scalar6)**(1/scalar3 -1) * (1+gama) * &

                              (cons**gama/scalar6) - (1 + phi*( kap**gama + cons*gama*kap**(gama-1)*kap1  ) )

                        cons = cons - fc/fc1

                end do

                end if

                kap  = cons**((1-varrho)/scalar3)*scalar6**(varrho/scalar3)
                hour = ( scalar5/( psi*(1+scalar2*kap**gama) ) )**scalar1           ! Computes the labor supply for a given real wage and productivity state

                    if ( (cons-psi*hour**eta/eta) <=0  ) then

                        Viter(i,j,ii)= -1.e+08                             ! Imposing Inada condition
                        Citer(i,j,ii) = 0.000001

                    elseif (sigma > 1.) then

                        Viter(i,j,ii) = ( (cons - psi*hour**eta/eta)**(1-sigma) - 1 ) / (1-sigma)
                        Citer(i,j,ii) = cons
                        mon = (cons**(1+gama)/scalar6)**(1/scalar3)
                        aprime = R_star_t(1,t)*bond(ii,1) + mon/(1+inf_t(1,t+1)) + Rk_a_x_t(1,t)     ! Evolution of wealth

                            if (aprime >= aub) then
                                aprime = aub
                            elseif (aprime <= alb) then
                                aprime = alb
                            end if

                        Lprime = int( (aprime - alb)/inca ) + 1          ! Indices of left shoulders
                        Rprime = Lprime + 1
                        LWprime = ( asset(Rprime,1) - aprime ) / inca    ! Computation of weights for tomorrow's wealth

                            if (LWprime > 1.) then
                                LWprime = 1.
                            elseif (LWprime < 0.) then
                                LWprime = 0.
                            end if

                        RWprime = 1. - LWprime

                    end if

                    do tt = 1,nx
                        Viter(i,j,ii) = Viter(i,j,ii) + beta*xprob(j,tt)*( LWprime*Vfuture(Lprime,tt) + &   ! Updating return function

                                                                           RWprime*Vfuture(Rprime,tt) )
                    end do

            end do    ! Closes the loop on ii =1,nb

            Vfuture(i,j) = maxval(Viter(i,j,:))

            I_bond(i,j)   = maxloc(Viter(i,j,:),1)

            b_policy_t(i,j,t) = bond(I_bond(i,j),1)

            c_policy_t(i,j,t) = Citer(i,j,I_bond(i,j))

            kap_policy_t(i,j,t) = c_policy_t(i,j,t)**((1-varrho)/scalar3)*scalar6**(varrho/scalar3)

            h_policy_t(i,j,t) = ( scalar5/( psi*(1+phi*(1+gama)*kap_policy_t(i,j,t)**gama) ) )**scalar1

            m_policy_t(i,j,t) = (c_policy_t(i,j,t)/kap_policy_t(i,j,t))**(1/varrho)

            a_policy_t(i,j,t) = R_star_t(1,t)*b_policy_t(i,j,t) + m_policy_t(i,j,t)/(1+inf_t(1,t+1)) + Rk_a_x_t(1,t)

            I_bond(i,j) = 1

    end do      ! Closes the loop on j =1,nx
end do          ! Closes the loop on i =1,na

Viter  = -1.e+08
Citer  = 0.0001

!!!!-------------------------------------------------------------------------------
END DO          ! Closes the loop on time
!!!!-------------------------------------------------------------------------------

print*,'------------------------------------------------'
print*,'Time dependent decision rules have been solved!'
print*,'------------------------------------------------'

end subroutine FinDecision
