MODULE parameters

!----------------------------------------------
!! Precision parameters
!----------------------------------------------

use osl_core, wp => osl_wp

INTEGER, PARAMETER :: DP = KIND(1.0D0)

INTEGER, PARAMETER :: SP = KIND(1.0)

!----------------------------------------------
!! Earnings process
!----------------------------------------------

integer, parameter :: nx = 3                                ! Number of earnings grid points

real(wp), parameter :: beta =  0.98137, delta = 0.0375, theta = 0.931 ! for the period 1996-2014 share of capital in production, discount factor, depreciation rate, returns-to-scale parameter

! Together with theta=.9, alpha implies a capital income share of 40%

! beta = 0.95432 when R_star depends on disinflation

real(wp), parameter :: sigma = 2. ,  psi = 70, varrho = 0.275 !    Risk aversion,  multiplicative of labor supply, decreases labor supply

! 0.9731413 is the upper bound for beta

real(wp), parameter :: cover = 3._dp                        ! Coverage of normal distribution

real(wp), dimension(nx,1) :: z                              ! Log of earnings points

real(wp), dimension(nx,nx) :: xprob                     ! Transition matrix

real(DP),dimension(nx,1) :: x,invx,invx1,difx               ! Earnings grid, invariant distribution of earnings process

real(DP) :: alpha = 1- .62/theta, var_x,sigma_x, x_bar       ! Coefficient of variation of the earnings process, mean of the earnings process, standard deviation of shocks

character:: input

integer, parameter ::  na = 400, n_asset = 5, nb = 2000, nm = n_asset*na, TD=28, T1 = 1, T2 = T1+TD, T3 = T2+501, T4 = T3+70    ! T3 = T2+501, T4 = T3+70

integer :: i,j,t,ii,jj,kk,hh,vv,n,counter,iteration,tt,stationary

integer,dimension(na,nx):: indmon,indbond,indcap,indasset,indhour

integer,dimension(na,nx):: I_money,I_bond,I_hour,I_vel,I_asset ! Indicator functions for policies

integer,dimension(na,nx)::LL,RR

real(DP),dimension(na,nx)::LW,RW

real,dimension(2)::values

real(DP),dimension(3)::enes

real::time

real(DP) ::phi,gama,eta,inf,inca,incm,inch,incv,incb,inck,incc,inccdf,inclorenz,bh,bl,tol1,bss,mss

real(DP) ::  tau,tau_k,tau_h,R_star,R_k,w,r_tilde,w_tilde,w1,w2,tau1,tau2,g1,g2,PS,fc1,CEV,rB,y_median,taudif,wdif

real(DP):: C,H,K,L,Y,A,B,M,G,k_l,TR,AY,AY_p,vlb,vub,hlb,hub,mlb,mub,blb,bub,klb,kub,alb,aub,VFI_check,clb,cub,vel,&
Anorm,H_L,K_H_next,Q,L_a_x,K_a_x,R_k_K_a_x

real(DP),dimension(na,nx) :: ypdf,y_disp

real(DP),dimension(na*nx) :: y_sorted,ypdf_sorted,m_sorted,mpdf_sorted,b_sorted,bpdf_sorted,k_sorted,kpdf_sorted,c_sorted,&
cpdf_sorted,h_sorted,hpdf_sorted,earn_inc,asset_inc,trans_inc,invasset_col,b_policy_col,m_policy_col,c_policy_col,&
kap_policy_col,h_policy_col,a_policy_col,value_col,y_col

real(DP),dimension((na*nx+1)*8) ::PI_col

real(DP),dimension(1,17) :: aggregates

real(DP),dimension(nb,1) :: bond,normb

real(DP),dimension(nm,1) :: money

real(DP),dimension(nb/2,1) :: bond1,bond2

real(DP),dimension(nm/2,1) :: money1,money2

real(DP),dimension(na,1) ::asset,norma

real(DP),dimension(na+1,1) :: acdf,alorenz

real(DP),dimension(5,1)::mtar,btar,ctar,ytar,etar,astar,transtar,portshare

integer ::Lprime,Rprime

real(DP)::aprime,lwprime,rwprime,upd_share1,upd_share

real(DP),dimension(na,nx):: V, Vnext, diff, invasset,trans,Policy,invasset_0,invasset_T4,V_T4,Vfuture,V_0

real(DP),dimension(na,nx,nb):: Viter,Citer

real(DP)::cons,u,mon,hour,labor,C_0,H_0,K_0,L_0,G_0,Tr_0,B_0,M_0,Y_0,C_T4,H_T4,K_T4,L_T4,G_T4,vel_0,&
Tr_T4,M_T4,Y_T4,B_T4,vel_T4,taustar_0,taustar_T4,gstar_0,gstar_T4,wstar_0,wstar_T4,Q_t_0,R_star_0,y_disp_0,&
kap,kap1,h1,fc,R_star_T4

real(DP),dimension(na,nx):: b_policy,m_policy,a_policy,c_policy,h_policy,b_policy_0,m_policy_0,a_policy_0,&
c_policy_0,kap_policy_0,h_policy_0,b_policy_T4,m_policy_T4,a_policy_T4,c_policy_T4,kap_policy_T4,h_policy_T4,b_dev

!!!----------------------------------------------
!! Variables regarding transition
!!!----------------------------------------------

real(DP),dimension(na,nx,T4):: b_policy_t,m_policy_t,a_policy_t,c_policy_t,h_policy_t,assetdist,ypdf_t,y_disp_t,&
kap_policy_t

real(DP),dimension(na*nx + 1,T4) :: ycdf_t,ylorenz_t,m_cdf_t,m_lorenz_t,b_cdf_t,b_lorenz_t,c_cdf_t,c_lorenz_t,&
fin_cdf_t,fin_lorenz_t

real(DP),dimension((na*nx+1),8,T4) ::PI_t

real(DP),dimension(na*nx,T4) :: y_sorted_t,ypdf_sorted_t,m_sorted_t,mpdf_sorted_t,b_sorted_t,bpdf_sorted_t,c_sorted_t,&
cpdf_sorted_t,fin_sorted_t,finpdf_sorted_t

real(DP),dimension(na*nx*T4) :: Bt_prime,Ct_prime,Ht_prime,Mt_prime,At_prime,At_dist

real(DP),dimension(1,T4)::AY_t,C_t,H_t,K_t,I_t,SE_t,L_t,Y_t,B_t,M_t,G_t,TR_t,tau_t,w_t,tau_t_next,taudiff_t,tau_k_t,&
tau_h_t,w_t_next,wdiff_t,inf_t,R_star_t,ED_t,L_H_t,PS_t,tau_1_t,tau1_t,tau2_t,infprime_t,G_t_next,Gdiff_t,K_a_x_t,Q_t,&
vel_t,L_a_x_t,Rk_a_x_t,gini_wealth_t,gini_y_t,gini_m_t,gini_b_t,gini_fin_t,gini_c_t ! Aggregates and distributional variables along the transition dynamics

real(DP)::first,second,third,fourth,timesum,scalar1,scalar2,scalar3,scalar4,scalar5,scalar6,scalar7,scalar8,scalar9

END MODULE
