! This subprogram generates the inequality measures for wealth, real balances holdings and bond holdings
! Written by Enes Sunel
! Central Bank of Turkey, January 2015
  subroutine Inequality
use parameters
implicit none

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Total Assets
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
acdf(1,1) = 0.

norma = asset - minval(asset)                ! We normalize the minimum wealth to zero

! Cumulative distribution of wealth
!-----------------------------------------------------
do i= 2,na + 1
   acdf(i,1) = acdf(i-1,1)
    do j=1,nx
   acdf(i,1) = acdf(i,1) + invasset(i-1,j)
    end do
end do
!-----------------------------------------------------

Anorm = 0.                               ! Total normalized wealth

!-----------------------------------------------------
alorenz(1,1)=0.

do i=2,na+1

alorenz(i,1)= alorenz(i-1,1)                 ! Lorenz curve for wealth

    do j=1,nx

alorenz(i,1)= alorenz(i,1) + norma(i-1,1)*invasset(i-1,j)

Anorm = Anorm + norma(i-1,1)*invasset(i-1,j) ! Aggregate normalized wealth

    end do

end do

alorenz = alorenz/Anorm

!------------------------------------------------------------------------------------------
n = 100
gini_wealth = 0.
do i=2,na+1

    gini_wealth = gini_wealth + ( acdf(i,1)-acdf(i-1,1) )*alorenz(i-1,1)

    do kk = 1,n-1

gini_wealth = gini_wealth + ( (n-kk)*(acdf(i,1) - acdf(i-1,1))/n ) * ( (alorenz(i,1)-alorenz(i-1,1))/n )

    end do      ! Approximating the area under the Lorenz curve


end do

gini_wealth = 2.*(0.5-gini_wealth)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!TOTAL DEPOSITS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
fin_sorted = reshape(b_policy + m_policy,(/na*nx/))

call quicksort(fin_sorted,size(fin_sorted))

fin_cdf(1,1) = 0.

fin_lorenz(1,1) = 0.

finpdf_sorted = 0.                             ! Probability density of the sorted bond grid

fintar = 0.

! Cumulative distribution and lorenz curve for bonds
!-----------------------------------------------------

do t = 1,na*nx

        if (t>1 .and. fin_sorted(t) == fin_sorted(t-1) ) then

        fin_cdf(t+1,1) = fin_cdf(t,1)
        fin_lorenz(t+1,1) = fin_lorenz(t,1)

        else

            do i=1,na
                do j=1,nx
                    if (  fin_sorted(t)  == ( b_policy(i,j) + m_policy(i,j) ) ) then
                        finpdf_sorted(t) = finpdf_sorted(t) + invasset(i,j)
                    end if
                end do
            end do

        fin_cdf(t+1,1) = fin_cdf(t,1) + finpdf_sorted(t)
        fin_lorenz(t+1,1) = fin_lorenz(t,1) + ( fin_sorted(t) - minval(fin_sorted)) *finpdf_sorted(t)     ! We use the normalized bond grid points while computing the Lorenz curve

        end if
end do

fin_lorenz = fin_lorenz/sum((fin_sorted-minval(fin_sorted))*finpdf_sorted)            ! We use the aggregate normalized bond while computing the Lorenz curve

!------------------------------------
! TARGETS ON LORENZ CURVE FOR TOTAL DEPOSITS
!------------------------------------

do t=2,na*nx+1
  if (fin_cdf(t,1)> .7045 .and. fin_cdf(t-1,1) < .7045) then

    lwprime = (  fin_cdf(t,1) - .7045 ) / ( fin_cdf(t,1) - fin_cdf(t-1,1) )

    fintar(1,1) = lwprime*fin_lorenz(t-1,1) + (1-lwprime)*fin_lorenz(t,1)         ! Our calibration target for the debt limit
    j=t
    exit
   end if

end do

do t=j-1,na*nx+1
  if (fin_cdf(t,1) > .9161 .and. fin_cdf(t-1,1) < .9161) then

    lwprime = (  fin_cdf(t,1) - .9161 ) / ( fin_cdf(t,1) - fin_cdf(t-1,1) )

    fintar(2,1) = lwprime*fin_lorenz(t-1,1) + (1-lwprime)*fin_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
  end if
end do

do t=j-1,na*nx+1
  if (fin_cdf(t,1)> .9873 .and. fin_cdf(t-1,1) < .9873) then

    lwprime = (  fin_cdf(t,1) - .9873 ) / ( fin_cdf(t,1) - fin_cdf(t-1,1) )

    fintar(3,1) = lwprime*fin_lorenz(t-1,1) + (1-lwprime)*fin_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
   end if
end do

do t=j-1,na*nx+1
  if (fin_cdf(t,1)> .9969 .and. fin_cdf(t-1,1) < .9969) then

    lwprime = (  fin_cdf(t,1) - .9969 ) / ( fin_cdf(t,1) - fin_cdf(t-1,1) )

    fintar(4,1) = lwprime*fin_lorenz(t-1,1) + (1-lwprime)*fin_lorenz(t,1)

    fintar(5,1) = 1

    exit
   end if
end do

!------------------------------------
! Gini coefficient of total deposits
!------------------------------------
n=100
gini_fin = 0.
do i=2,na*nx+1

gini_fin = gini_fin + (fin_cdf(i,1)-fin_cdf(i-1,1))*fin_lorenz(i-1,1)

    do kk = 1,n-1

gini_fin = gini_fin + ( (n-kk)*(fin_cdf(i,1) - fin_cdf(i-1,1))/n ) * ( (fin_lorenz(i,1)-fin_lorenz(i-1,1))/n )

    end do      ! Approximating the area under the Lorenz curve

end do

gini_fin = 2.*(0.5-gini_fin)


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!BONDS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
b_sorted = reshape(b_policy,(/na*nx/))

call quicksort(b_sorted,size(b_sorted))

b_cdf(1,1) = 0.

b_lorenz(1,1) = 0.

bpdf_sorted = 0.                             ! Probability density of the sorted bond grid

btar = 0.

! Cumulative distribution and lorenz curve for bonds
!-----------------------------------------------------

do t = 1,na*nx

        if (t>1 .and. b_sorted(t) == b_sorted(t-1) ) then

        b_cdf(t+1,1) = b_cdf(t,1)
        b_lorenz(t+1,1) = b_lorenz(t,1)

        else

            do i=1,na
                do j=1,nx
                    if ( (b_sorted(t)) == b_policy(i,j)) then
                        bpdf_sorted(t) = bpdf_sorted(t) + invasset(i,j)
                    end if
                end do
            end do

        b_cdf(t+1,1) = b_cdf(t,1) + bpdf_sorted(t)
        b_lorenz(t+1,1) = b_lorenz(t,1) + ( b_sorted(t) - minval(b_sorted)) *bpdf_sorted(t)     ! We use the normalized bond grid points while computing the Lorenz curve

        end if
end do

b_lorenz = b_lorenz/sum((b_sorted-minval(b_sorted))*bpdf_sorted)            ! We use the aggregate normalized bond while computing the Lorenz curve

!------------------------------------
! TARGETS ON LORENZ CURVE FOR BONDS
!------------------------------------

do t=2,na*nx+1
  if (b_cdf(t,1)> .7045 .and. b_cdf(t-1,1) < .7045) then

    lwprime = (  b_cdf(t,1) - .7045 ) / ( b_cdf(t,1) - b_cdf(t-1,1) )

    btar(1,1) = lwprime*b_lorenz(t-1,1) + (1-lwprime)*b_lorenz(t,1)         ! Our calibration target for the debt limit
    j=t
    exit
   end if

end do

do t=j-1,na*nx+1
  if (b_cdf(t,1) > .9161 .and. b_cdf(t-1,1) < .9161) then

    lwprime = (  b_cdf(t,1) - .9161 ) / ( b_cdf(t,1) - b_cdf(t-1,1) )

    btar(2,1) = lwprime*b_lorenz(t-1,1) + (1-lwprime)*b_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
  end if
end do

do t=j-1,na*nx+1
  if (b_cdf(t,1)> .9873 .and. b_cdf(t-1,1) < .9873) then

    lwprime = (  b_cdf(t,1) - .9873 ) / ( b_cdf(t,1) - b_cdf(t-1,1) )

    btar(3,1) = lwprime*b_lorenz(t-1,1) + (1-lwprime)*b_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
   end if
end do

do t=j-1,na*nx+1
  if (b_cdf(t,1)> .9969 .and. b_cdf(t-1,1) < .9969) then

    lwprime = (  b_cdf(t,1) - .9969 ) / ( b_cdf(t,1) - b_cdf(t-1,1) )

    btar(4,1) = lwprime*b_lorenz(t-1,1) + (1-lwprime)*b_lorenz(t,1)

    btar(5,1) = 1

    exit
   end if
end do

!------------------------------------
! Gini coefficient of bonds
!------------------------------------
n=100
gini_b = 0.
do i=2,na*nx+1

gini_b = gini_b + (b_cdf(i,1)-b_cdf(i-1,1))*b_lorenz(i-1,1)

    do kk = 1,n-1

gini_b = gini_b + ( (n-kk)*(b_cdf(i,1) - b_cdf(i-1,1))/n ) * ( (b_lorenz(i,1)-b_lorenz(i-1,1))/n )

    end do      ! Approximating the area under the Lorenz curve

end do

gini_b = 2.*(0.5-gini_b)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!REAL BALANCES
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

M_sorted = reshape(m_policy,(/na*nx/))

call quicksort(m_sorted,size(m_sorted))

m_cdf(1,1) = 0.

m_lorenz(1,1) = 0.

mpdf_sorted = 0.

mtar = 0.


! Cumulative distribution and lorenz curve for real balances
!-------------------------------------------------------------

do t = 1,na*nx

        if (t>1 .and. m_sorted(t) == m_sorted(t-1) ) then

        m_cdf(t+1,1) = m_cdf(t,1)
        m_lorenz(t+1,1) = m_lorenz(t,1)

        else

            do i=1,na
                do j=1,nx
                    if (m_sorted(t) == m_policy(i,j)) then
                        mpdf_sorted(t) = mpdf_sorted(t) + invasset(i,j)
                    end if
                end do
            end do

        m_cdf(t+1,1) = m_cdf(t,1) + mpdf_sorted(t)
        m_lorenz(t+1,1) = m_lorenz(t,1) + m_sorted(t)*mpdf_sorted(t)

        end if
end do

m_lorenz = m_lorenz/M

do t=2,na*nx+1
  if (m_cdf(t,1)> .9732 .and. m_cdf(t-1,1) < .9732) then

    lwprime = (  m_cdf(t,1) - .9734 ) / ( m_cdf(t,1) - m_cdf(t-1,1) )

    mtar(1,1) = lwprime*m_lorenz(t-1,1) + (1-lwprime)*m_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
  end if
end do

do t=j-1,na*nx+1

   if (m_cdf(t,1)> .9931 .and. m_cdf(t-1,1) < .9931) then

    lwprime = (  m_cdf(t,1) - .9931 ) / ( m_cdf(t,1) - m_cdf(t-1,1) )

    mtar(2,1) = lwprime*m_lorenz(t-1,1) + (1-lwprime)*m_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
   end if
end do

do t=j-1,na*nx+1

      if (m_cdf(t,1)> .999 .and. m_cdf(t-1,1) < .999) then

    lwprime = (  m_cdf(t,1) - .999 ) / ( m_cdf(t,1) - m_cdf(t-1,1) )

    mtar(3,1) = lwprime*m_lorenz(t-1,1) + (1-lwprime)*m_lorenz(t,1)         ! Our calibration target for the debt limit

    j=t

    exit
   end if
end do

do t=j-1,na*nx+1

      if (m_cdf(t,1)> .9998 .and. m_cdf(t-1,1) < .9998) then

    lwprime = (  m_cdf(t,1) - .9998 ) / ( m_cdf(t,1) - m_cdf(t-1,1) )

    mtar(4,1) = lwprime*m_lorenz(t-1,1) + (1-lwprime)*m_lorenz(t,1)         ! Our calibration target for the debt limit

    mtar(5,1) = 1

    exit
   end if
end do

do i=1,5
portshare(i,1) = btar(i,1)*B / ( btar(i,1)*B + mtar(i,1)*M )
end do

PRINT*,'---------------------------------------------------------------------'
print*,'Targets on bonds',btar(1,1),btar(2,1),btar(3,1),btar(4,1),btar(5,1)
PRINT*,'---------------------------------------------------------------------'
print*,'Targets on money',mtar(1,1),mtar(2,1),mtar(3,1),mtar(4,1),mtar(5,1)
print*,'------------------------------------------------------------------------'
print*,'ENES :', sum(btar),sum(mtar)


!------------------------------------
! Gini coefficient of real balances
!------------------------------------
n=100
gini_m = 0.
do i=2,na*nx+1

gini_m = gini_m + (m_cdf(i,1)-m_cdf(i-1,1))*m_lorenz(i-1,1)

    do kk = 1,n-1

gini_m = gini_m + ( (n-kk)*(m_cdf(i,1) - m_cdf(i-1,1))/n ) * ( (m_lorenz(i,1)-m_lorenz(i-1,1))/n )

    end do      ! Approximating the area under the Lorenz curve

end do

gini_m = 2.*(0.5-gini_m)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!CONSUMPTION
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c_sorted = reshape(c_policy,(/na*nx/))

call quicksort(c_sorted,size(c_sorted))

c_cdf(1,1) = 0.

c_lorenz(1,1) = 0.

cpdf_sorted = 0.

! Cumulative distribution and lorenz curve for consumption
!-----------------------------------------------------

do t = 1,na*nx

        if (t>1 .and. c_sorted(t) == c_sorted(t-1) ) then

        c_cdf(t+1,1) = c_cdf(t,1)
        c_lorenz(t+1,1) = c_lorenz(t,1)

        else

            do i=1,na
                do j=1,nx
                    if (c_sorted(t) == c_policy(i,j)) then
                        cpdf_sorted(t) = cpdf_sorted(t) + invasset(i,j)
                    end if
                end do
            end do

        c_cdf(t+1,1) = c_cdf(t,1) + cpdf_sorted(t)
        c_lorenz(t+1,1) = c_lorenz(t,1) + c_sorted(t)*cpdf_sorted(t)

        end if
end do

c_lorenz = c_lorenz/C

do i=2,na*nx+1
  if (c_cdf(i,1)> .20 .and. c_cdf(i-1,1) < .20) then

    lwprime = ( c_cdf(i,1) - .20 ) / ( c_cdf(i,1) - c_cdf(i-1,1) )

    ctar(1,1) = lwprime*c_lorenz(i-1,1) + (1-lwprime)*c_lorenz(i,1)

    j = i

    exit
    end if
end do

do i = j-1,na*nx+1
  if (c_cdf(i,1)> .40 .and. c_cdf(i-1,1) < .40) then

    lwprime = ( c_cdf(i,1) - .40 ) / ( c_cdf(i,1) - c_cdf(i-1,1) )

    ctar(2,1) = lwprime*c_lorenz(i-1,1) + (1-lwprime)*c_lorenz(i,1) - ctar(1,1) ! Shares of consumption quintiles

    j = i

    exit
    end if
end do

do i = j-1,na*nx+1
  if (c_cdf(i,1)> .60 .and. c_cdf(i-1,1) < .60) then

    lwprime = ( c_cdf(i,1) - .60 ) / ( c_cdf(i,1) - c_cdf(i-1,1) )

    ctar(3,1) = lwprime*c_lorenz(i-1,1) + (1-lwprime)*c_lorenz(i,1) - (ctar(1,1)+ctar(2,1))

    j = i

    exit
    end if
end do


do i=j-1,na*nx+1
  if (c_cdf(i,1) > .80 .and. c_cdf(i-1,1) < .80) then

    lwprime = ( c_cdf(i,1) - .80 ) / ( c_cdf(i,1) - c_cdf(i-1,1) )

    ctar(4,1) = lwprime*c_lorenz(i-1,1) + (1-lwprime)*c_lorenz(i,1) - (ctar(1,1)+ctar(2,1)+ctar(3,1))

    j = i

    exit
    end if
end do

    ctar(5,1) = 1 - sum(ctar(1:4,1),1)

c_median  = 0.

do t=2,na*nx + 1
  if (c_cdf(t,1)> .50 .and. c_cdf(t-1,1) < .50) then

    lwprime = ( c_cdf(t,1) - .50 ) / ( c_cdf(t,1) - c_cdf(t-1,1) )

    c_median = lwprime*c_sorted(t-1) + (1-lwprime)*c_sorted(t)          ! First quintile of income

    exit
    end if
end do

PRINT*,'---------------------------------------------------------------------'
print*,'Shares of consumption quintiles',ctar(1,1),ctar(2,1),ctar(3,1),ctar(4,1),ctar(5,1)

!------------------------------------
! Gini coefficient of consumption
!------------------------------------
n=100
gini_c = 0.
do i=2,na*nx+1

gini_c = gini_c + (c_cdf(i,1)-c_cdf(i-1,1))*c_lorenz(i-1,1)

    do kk = 1,n-1

gini_c = gini_c + ( (n-kk)*(c_cdf(i,1) - c_cdf(i-1,1))/n ) * ( (c_lorenz(i,1)-c_lorenz(i-1,1))/n )

    end do      ! Approximating the area under the Lorenz curve

end do

gini_c = 2.*(0.5-gini_c)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!INCOME
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
y_sorted = reshape(y_disp,(/na*nx/))

call quicksort(y_sorted,size(y_sorted))

ycdf(1,1) = 0.

ylorenz(1,1) = 0.

ypdf_sorted = 0.

!-----------------------------------------------------
! Cumulative distribution and lorenz curve for income
!-----------------------------------------------------

do kk = 1,na*nx

        if (kk>1 .and. y_sorted(kk) == y_sorted(kk-1) ) then

        ycdf(kk+1,1) = ycdf(kk,1)
        ylorenz(kk+1,1) = ylorenz(kk,1)

        else

            do i=1,na
                do j=1,nx
                    if (y_sorted(kk) == y_disp(i,j)) then

                        ypdf_sorted(kk) = ypdf_sorted(kk) + ypdf(i,j)

                        earn_inc(kk)  = (1-tau_h)*w*h_policy(i,j)*x(j,1)

                        asset_inc(kk) = (1-tau_k)*((K_a_x**alpha*L_a_x**(1.-alpha))**theta &

                                       - delta*K_a_x - w*L_a_x) +(R_star -1)*b_policy(i,j) &

                                       - inf*m_policy(i,j)/(1+inf)

                        trans_inc(kk) = tau(i,j)

                    end if
                end do
            end do

        ycdf(kk+1,1) = ycdf(kk,1) + ypdf_sorted(kk)
        ylorenz(kk+1,1) = ylorenz(kk,1) + y_sorted(kk)*ypdf_sorted(kk)

        end if
end do

ylorenz = ylorenz/AY

!!!-----------------------------------------------------
!!! TARGETS ON LORENZ CURVE FOR INCOME
!!!-----------------------------------------------------

etar = 0.
astar = 0.
transtar = 0.

do t=2,na*nx + 1

  etar(1,1) = etar(1,1) + earn_inc(t-1)*ypdf_sorted(t-1)
  astar(1,1) = astar(1,1) + asset_inc(t-1)*ypdf_sorted(t-1)
  transtar(1,1) = transtar(1,1) + trans_inc(t-1)*ypdf_sorted(t-1)

  if (ycdf(t,1)> .20 .and. ycdf(t-1,1) < .20) then

    lwprime = ( ycdf(t,1) - .20 ) / ( ycdf(t,1) - ycdf(t-1,1) )

    ytar(1,1) = lwprime*ylorenz(t-1,1) + (1-lwprime)*ylorenz(t,1)           ! First quintile of income

    etar(1,1) = etar(1,1)/(w*H)
    astar(1,1) = astar(1,1)/((R_star-1)*B)
    transtar(1,1) = transtar(1,1)/sum(tau*invasset)
    j = t

    exit
  end if
end do

do t=j-1,na*nx + 1

  etar(2,1) = etar(2,1) + earn_inc(t-1)*ypdf_sorted(t-1)
  astar(2,1) = astar(2,1) + asset_inc(t-1)*ypdf_sorted(t-1)
  transtar(2,1) = transtar(2,1) + trans_inc(t-1)*ypdf_sorted(t-1)

  if (ycdf(t,1)> .40 .and. ycdf(t-1,1) < .40) then

    lwprime = ( ycdf(t,1) - .40 ) / ( ycdf(t,1) - ycdf(t-1,1) )

    ytar(2,1) = lwprime*ylorenz(t-1,1) + (1-lwprime)*ylorenz(t,1)  - ytar(1,1)

    etar(2,1) = etar(2,1)/(w*H)
    astar(2,1) = astar(2,1)/((R_star-1.)*B)
    transtar(2,1) = transtar(2,1)/sum(tau*invasset)
    j = t

    exit
  end if
end do

do t=j-1,na*nx + 1

  etar(3,1) = etar(3,1) + earn_inc(t-1)*ypdf_sorted(t-1)
  astar(3,1) = astar(3,1) + asset_inc(t-1)*ypdf_sorted(t-1)
  transtar(3,1) = transtar(3,1) + trans_inc(t-1)*ypdf_sorted(t-1)

  if (ycdf(t,1)> .60 .and. ycdf(t-1,1) < .60) then

    lwprime = ( ycdf(t,1) - .60 ) / ( ycdf(t,1) - ycdf(t-1,1) )

    ytar(3,1) = lwprime*ylorenz(t-1,1) + (1-lwprime)*ylorenz(t,1)  - (ytar(1,1)+ytar(2,1))

    etar(3,1) = etar(3,1)/(w*H)
    astar(3,1) = astar(3,1)/((R_star-1.)*B)
    transtar(3,1) = transtar(3,1)/sum(tau*invasset)
    j = t

    exit
  end if
end do

do t=j-1,na*nx + 1

  etar(4,1) = etar(4,1) + earn_inc(t-1)*ypdf_sorted(t-1)
  astar(4,1) = astar(4,1) + asset_inc(t-1)*ypdf_sorted(t-1)
  transtar(4,1) = transtar(4,1) + trans_inc(t-1)*ypdf_sorted(t-1)

  if (ycdf(t,1)> .80 .and. ycdf(t-1,1) < .80) then

    lwprime = ( ycdf(t,1) - .80 ) / ( ycdf(t,1) - ycdf(t-1,1) )

    ytar(4,1) = lwprime*ylorenz(t-1,1) + (1-lwprime)*ylorenz(t,1)  - (ytar(1,1)+ytar(2,1)+ytar(3,1))

    etar(4,1) = etar(4,1)/(w*H)
    astar(4,1) = astar(4,1)/((R_star-1.)*B)
    transtar(4,1) = transtar(4,1)/sum(tau*invasset)

    etar(5,1) = sum(earn_inc(t:na*nx)*ypdf_sorted(t:na*nx))/(w*H)
    astar(5,1) = sum(asset_inc(t:na*nx)*ypdf_sorted(t:na*nx))/((R_star-1)*B)
    transtar(5,1) = sum(trans_inc(t:na*nx)*ypdf_sorted(t:na*nx))/sum(tau*invasset)
    j = t

    exit
  end if
end do

    ytar(5,1) = 1 -sum(ytar(1:4,1),1)

PRINT*,'---------------------------------------------------------------------'
print*,'Shares of income quintiles',ytar(1,1),ytar(2,1),ytar(3,1),ytar(4,1),ytar(5,1)
!PRINT*,'---------------------------------------------------------------------'
!print*,'Earnings share of income quintiles',etar
!PRINT*,'---------------------------------------------------------------------'
!print*,'Asset income share of income quintiles',astar
!PRINT*,'---------------------------------------------------------------------'
!print*,'Transfers share of income quintiles',transtar

y_median  = 0.

do t=2,na*nx + 1
  if (ycdf(t,1)> .50 .and. ycdf(t-1,1) < .50) then

    lwprime = ( ycdf(t,1) - .50 ) / ( ycdf(t,1) - ycdf(t-1,1) )

    y_median = lwprime*y_sorted(t-1) + (1-lwprime)*y_sorted(t)          ! First quintile of income

    exit
    end if
end do

!!!--------------------------------
!!! Gini coefficient of income
!!!--------------------------------

n=100
gini_y = 0.
do i=2,na*nx+1

gini_y = gini_y + (ycdf(i,1)-ycdf(i-1,1))*ylorenz(i-1,1)

    do kk = 1,n-1

gini_y = gini_y + ( (n-kk)*(ycdf(i,1) - ycdf(i-1,1))/n ) * ( (ylorenz(i,1)-ylorenz(i-1,1))/n )

    end do      ! Approximating the area under the Lorenz curve

end do

gini_y = 2.*(0.5-gini_y)

print*,'---------------------------------------------------------------------'
print*,'Gini Coefficient of Total Deposits : ', gini_wealth
PRINT*,'---------------------------------------------------------------------'
print*,'Mean-Median Ratio of Consumption :', C/c_median
PRINT*,'---------------------------------------------------------------------'
print*,'Mean-Median Ratio of Income (data):', AY/y_median, 1.34_dp
PRINT*,'---------------------------------------------------------------------'
print*,'Top 20/Bottom 20 Ratio of Income (data):', (1. - ytar(2,1))/ytar(1,1), 8.0_dp
PRINT*,'---------------------------------------------------------------------'
print*,'Top 20/Bottom 20 Ratio of Consumption :', (1. - ctar(2,1))/ctar(1,1), 6.1429_dp
print*,'---------------------------------------------------------------------'
print*,'Portfolio composition of the first bin (data):', portshare(1,1), 0.6403_dp
print*,'------------------------------------------------------------------------'
print*,'97th percentile on the Lorenz curve for money (data):', mtar(1,1), 0.2111_dp
PRINT*,'---------------------------------------------------------------------'
print*,'Gini Coefficient of money (data):',gini_m, 0.77595_dp
print*,'---------------------------------------------------------------------'
print*,'Gini Coefficient of bonds (data):',gini_b, 0.78087_dp
print*,'---------------------------------------------------------------------'
print*,'Gini Coefficient of total deposits (data):',gini_fin, 0.85883_dp
PRINT*,'---------------------------------------------------------------------'
print*,'Gini Coefficient of Income (data): ', gini_y, 0.376_dp
PRINT*,'---------------------------------------------------------------------'
print*,'Gini Coefficient of Consumption :',gini_c, 0.328_dp
print*,'------------------------------------------------------------------------'

end subroutine Inequality
