! May 2014, Central Bank of Turkey
! This subprogram initializes parameterization and grids. The model economy is calibrated to the Turkish economy between 2004:1-2009:4
! Written by Enes Sunel

subroutine Initialize
use parameters
implicit none

!-------------------------
! Idiosyncratic earnings                        ! These values are taken from Ragot (2010)
!-------------------------
! call tauchen


x(1,1) = 4.74                                ! Productivity levels
x(2,1) = 0.848
x(3,1) = 0.17

x = x

xprob(1,1) = 0.974                           ! Productivity transition probabilities
xprob(1,2) = 0.026
xprob(1,3) = 0.
xprob(2,1) = 0.0013
xprob(2,2) = 0.9974
xprob(2,3) = 0.0013
xprob(3,1) = 0.
xprob(3,2) = 0.026
xprob(3,3) = 0.974

!---------------------
! Aggregate Earnings
!---------------------

  invx = 1./(nx)

  tol1 =1.e-06
  difx=10.
  do

   if (maxval(difx) <= tol1) exit

   invx1 = matmul(transpose(xprob),invx)

   difx = maxval(abs(invx1-invx))

   invx = invx1;                            ! Invariant distribution of earnings

  end do

  x_bar = sum(x*invx)                       ! Aggregate earnings

  var_x = 0.

  do i=1,nx

    var_x = var_x + invx(i,1)*(x(i,1)-x_bar)**2.

  end do

!  x = x_bar

!---------------------------------
! Other Parameters and Aggregates
!---------------------------------
eta  = 4.5                                    ! Power of hours in the utility function, increases labor supply, decreases asset position

gama = 1.85 ! 1.0576 !                                ! Implied by the curvature of the transactions cost function implied by the money demand estimate for 1987:1-2014:3 and the varrho

phi  = 0.0052                               ! Multiplicative parameter of the transactions cost function

R_star = 1.0158                                     ! International real interest rate R = 1.0398 for 1996-2002 R = 1.0012 for 2003-2014 and R = 1.0158 for 1996-2014 1.0276

inf   = 0.02   !    0.14   !                                                                 ! Average quarterly CPI inflation rate 2003:1-2014:3 for low, 1987:1-2002:4 high

tau_k = 0.225                                       ! Average corporate tax rate between 2003 and 2014 calibrated from data

tau_h = 0.21                                        ! Average personal labor income tax burden between 2000 and 2013 (Source: OECD)

!! RIGHT AND LEFT SHOULDERS FOR EQUILIBRIUM TRANSFERS LEVEL AND REAL WAGE

if (inf == 0.02) then

!!!!! Inflation LOW !!!!

tau1 =  3.30E-002 !   3.90E-002                        ! Negative public surplus generating transfers level

tau2 =  3.09E-002     !   3.1890625192318112E-002                     ! Positive public surplus generating transfers level

w1 =    1.48200        !   1.4797265399247408                            ! Real wage that produces excess supply of labor

elseif (inf == .14 ) then

!!!!! Inflation HIGH !!!!

tau1 =  5.3749999962747097E-002 ! 6.50E-002     !   3.1843750155530870E-002                ! Negative public surplus generating transfers level

tau2 =  5.00E-002     !   3.1843750155530870E-002                ! Positive public surplus generating transfers level

w1 =  1.4812500178813934    !  1.4850          !   1.4800781048834324                         ! Real wage that produces positive excess supply of labor

end if

w2 = ( (1-alpha)*theta*( (alpha/(1-alpha)) / ((R_star-1)/(1-tau_k) + delta) )**(alpha*theta)   )**(1/(1-alpha*theta)) ! Real wage cannot be smaller than this value to obtain L_a_x < 1

w2 =  1.48097        ! 1.4775  !                      ! Real wage that produces excess demand for labor

!!!!!!!!!!!!!!!!!!!!!!!! Initializing Candidate Equilibrium Objects Depending on the Experiment !!!!!!!!!!!!!!!!!!!!!!!!

Print*,'W2 ENEEEES',w2

!!!  BENCHMARK ECONOMY

tau  = tau1

w = w1                                              ! Before-tax real wage rate, constant in stationary equilibrium, ITERATE ON THIS TO CLEAR THE LABOR MARKET

!!!!!!!!!!!!!!!!! Equilibrium Objects Dependent Variables !!!!!!!!!!!!!!!!!!!!!!!

Q = (alpha/(1-alpha))* w / ( (R_star-1)/(1-tau_k) + delta ) ! Homogeneous capital-labor demand ratio implied by the real interest rate, real wage rage and capital income tax rate

L_a_x = ( w/(theta*(1-alpha)*Q**(alpha*theta) )  )**(1/(theta-1)) ! Homogeneous labor demand for the given real wage

K_a_x = Q*L_a_x                                     ! Homogeneous capital demand for the given real wage

print*,'K_a_x',K_a_x

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

H = 0.33                                            ! Target for aggregate labor supply

Y = (K_a_x**alpha*0.33**(1.-alpha))**theta  ! 0.62132                                         ! Output

G = 0.133*0.7927    ! 0.17*Y                                          ! The exogenous government spending + government investment for the period 2002:1-2014:1 is 0.175*Y gov. spend/Y=13.7%

C = 0.72*Y                                          ! The average adjusted C/Y ratio for the period 2002:1-2014:1

TR = C*phi*(vel**gama)                              ! Initial transaction costs

M = C/2.955                                           ! Initial aggregate demand deposits

B = -0.045*Y/(1.-R_star)                            ! Initial aggregate bonds implied by the NX/Y ratio for the period 2002:1-2014:1

print*,tau/Y,'ENESS'

print*,'tau/Y: ',tau/Y,'Q: ',Q,'L_a_x: ',L_a_x,'K_a_x: ',K_a_x,'G: ', G,'w2: ', w2

iteration = 0

!--------------------------------------
! Grid for Demand Deposits
!--------------------------------------

mlb = 0.001                                      ! Lowest possible real balances value

mub = 5.00                                       ! Largest Possible real balances value

call grid(mlb,mub,nm,incm,money)                 ! Creates the money grid

!--------------------------------------
! Grid for Bonds
!--------------------------------------

blb = 0.                                       ! -0.75 !

bub = 15.

call grid(blb,bub,nb,incb,bond)                    ! Creates the bonds grid

!--------------------------------------
! Grid for Total Assets
!--------------------------------------

alb = R_star*blb ! -0.75 !

aub = 24.833276561088887

call grid(alb,aub,na,inca,asset)                   ! Creates the total assets grid

return

end Subroutine Initialize
